/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * vim: sw=2 ts=2 sts=2
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Places code
 *
 * The Initial Developer of the Original Code is
 * Mozilla Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Shawn Wilsher <me@shawnwilsher.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

/**
 * This interface provides some constants used by the Places AutoComplete
 * search provider.
 */
[scriptable, uuid(a5ae8332-333c-412a-bb02-a35df8247714)]
interface mozIPlacesAutoComplete : nsISupports
{
  //////////////////////////////////////////////////////////////////////////////
  //// Matching Constants

  /**
   * Match anywhere in each searchable term.
   */
  const long MATCH_ANYWHERE = 0;

  /**
   * Match first on word boundaries, and if we do not get enough results, then
   * match anywhere in each searchable term.
   */
  const long MATCH_BOUNDARY_ANYWHERE = 1;

  /**
   * Match on word boundaries in each searchable term.
   */
  const long MATCH_BOUNDARY = 2;

  /**
   * Match only the beginning of each search term.
   */
  const long MATCH_BEGINNING = 3;

  //////////////////////////////////////////////////////////////////////////////
  //// Search Behavior Constants

  /**
   * Search through history.
   */
  const long BEHAVIOR_HISTORY = 1 << 0;

  /**
   * Search though bookmarks.
   */
  const long BEHAVIOR_BOOKMARK = 1 << 1;

  /**
   * Search through tags.
   */
  const long BEHAVIOR_TAG = 1 << 2;

  /**
   * Search the title of pages.
   */
  const long BEHAVIOR_TITLE = 1 << 3;

  /**
   * Search the URL of pages.
   */
  const long BEHAVIOR_URL = 1 << 4;

  /**
   * Search for typed pages.
   */
  const long BEHAVIOR_TYPED = 1 << 5;

  /**
   * Search javascript: URLs.
   */
  const long BEHAVIOR_JAVASCRIPT = 1 << 6;

  /**
   * Search for pages that have been marked as being opened, such as a tab
   * in a tabbrowser, via:
   *   nsIBrowserHistory.registerOpenPage(url)
   *
   * @see nsIBrowserHistory.idl
   */
  const long BEHAVIOR_OPENPAGE = 1 << 7;
};
